@@ -66,6 +66,9 @@ void donotuseoldinstall(){
 //
 ////////////////////////////////////////////////////////////////////////
 
+subst::~subst(){
+}
+
 // subst_string
 subst_string::subst_string(string s){
   data=s;
@@ -76,13 +79,13 @@ string subst_string::get(menuentry *){
 
 // subst_var
 menuentry_f subst_var::str_to_menu_f(string f){
-  if     (f=="type")    return menuentry::get_type;
-  else if(f=="fullsection") return menuentry::fullsection;
-  else if(f=="section") return menuentry::get_subsec;
-  else if(f=="title")   return menuentry::get_title;
-  else if(f=="command") return menuentry::get_command;//returns precomm
-  else if(f=="id")      return menuentry::get_id;
-  else if(f=="icon")    return menuentry::get_icon;
+  if     (f=="type")    return &menuentry::get_type;
+  else if(f=="fullsection") return &menuentry::fullsection;
+  else if(f=="section") return &menuentry::get_subsec;
+  else if(f=="title")   return &menuentry::get_title;
+  else if(f=="command") return &menuentry::get_command;//returns precomm
+  else if(f=="id")      return &menuentry::get_id;
+  else if(f=="icon")    return &menuentry::get_icon;
   else{ 
     cerr<<"In config file: Unknown variable \""<<f<<"\""<<endl;
     exit(1);
