@@ -9,11 +9,13 @@
 #include <stdlib.h>
 #include <values.h>
 #include <sys/types.h>
+#include <dirent.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include "menu-method.h"
 #include <sys/stat.h>
 
+#include "menu-method.h"
+
 int show_time=0, verbose=0, dodebug=0;
 
 map <String, func *, less<String> > func_data;
@@ -43,10 +45,13 @@ void usage(){
   exit(1);
 }
 
-int store_func(func *f){
+func_def::~func_def(){
+}
+
+void store_func(func *f){
   func_data[f->name()]=f;
 }
-int add_functions(){
+void add_functions(){
   store_func(new prefix_func);
   store_func(new ifroot_func);
 
@@ -82,7 +87,7 @@ int add_functions(){
 
 
 bool empty_String(const String &s){
-  if(s.length()&&(s!="none"))
+  if(s.length()&&(s!=String("none")))
     return false;
   else
     return true;
@@ -103,7 +108,7 @@ int check_dir(String s){
   while(s.length()){
     i=s.index('/',1);
     if(i>=0){
-      t=s.before('/', 1)+'/';
+      t=s.before('/', 1)+String('/');
       for(; ((unsigned int)i<s.length()) && (s[i]=='/'); i++);
       s=s.after(i-1);
     } else {
@@ -156,7 +161,8 @@ void genoutput(const String &s,
 //  Construction:
 //
 
-
+str::~str(){
+}
 cat_str::cat_str(parsestream &i){
   char c;
   c=i.get_char();
@@ -602,7 +608,7 @@ supportedinfo::supportedinfo(parsestream &i){
       String name;
       i.skip_space();
       name=i.get_name();
-      if(name=="endsupported")
+      if(name==String("endsupported"))
 	return;
       name=upcase(name);
       if(dodebug)
@@ -670,60 +676,60 @@ configinfo::configinfo(parsestream &i){
     while(1){
         String name;
         name=i.get_name();
-	if(name=="supported"){
+	if(name==String("supported")){
 	  i.skip_line();
 	  supported=new supportedinfo(i);
 	}
-	else if(name=="function")
+	else if(name==String("function"))
 	  store_func(new func_def(i));
-	else if(name=="startmenu")
+	else if(name==String("startmenu"))
 	  startmenu=get_eq_cat_str(i);
-	else if(name=="endmenu")
+	else if(name==String("endmenu"))
 	  endmenu=get_eq_cat_str(i);
-	else if(name=="submenutitle")
+	else if(name==String("submenutitle"))
 	  submenutitle=get_eq_cat_str(i);
-	else if(name=="hotkeyexclude")
+	else if(name==String("hotkeyexclude"))
 	  hkexclude=get_eq_cat_str(i);
-	else if(name=="genmenu")
+	else if(name==String("genmenu"))
 	  genmenu=get_eq_cat_str(i);	
-	else if(name=="postrun")
+	else if(name==String("postrun"))
 	  postrun=get_eq_cat_str(i);	
-	else if(name=="prerun")
+	else if(name==String("prerun"))
 	  prerun=get_eq_cat_str(i);	
 
-	else if(name=="compat"){
-	  if((compt=i.get_eq_Stringconst())!="menu-1"){
+	else if(name==String("compat")){
+	  if((compt=i.get_eq_Stringconst())!=String("menu-1")){
 	    cerr<<"Unknown compat mode "<<compt<<" in config file (only know of \""COMPAT_MODE"\"";
 	    throw informed_fatal();
 	  }
 	}
-	else if(name=="rcfile")
+	else if(name==String("rcfile"))
 	  rcf=i.get_eq_Stringconst();
-	else if(name=="examplercfile")
+	else if(name==String("examplercfile"))
 	  exrcf=i.get_eq_Stringconst();
-	else if(name=="mainmenutitle")
+	else if(name==String("mainmenutitle"))
 	  mainmt=i.get_eq_Stringconst();
-	else if(name=="rootsection")
+	else if(name==String("rootsection"))
 	  roots=i.get_eq_Stringconst();
-	else if(name=="rootprefix")
+	else if(name==String("rootprefix"))
 	  rootpref=i.get_eq_Stringconst();
-	else if(name=="userprefix")
+	else if(name==String("userprefix"))
 	  userpref=i.get_eq_Stringconst();
-	else if(name=="treewalk")
+	else if(name==String("treewalk"))
 	  treew=i.get_eq_Stringconst();
-	else if(name=="postoutput")
+	else if(name==String("postoutput"))
 	  postout=i.get_eq_Stringconst();
-	else if(name=="preoutput")
+	else if(name==String("preoutput"))
 	  preout=i.get_eq_Stringconst();
-	else if(name=="command"){
+	else if(name==String("command")){
 	  system(i.get_eq_Stringconst());
 	  exit(0);
 	}
-	else if(name=="hotkeycase"){
+	else if(name==String("hotkeycase")){
 	  String s=i.get_eq_Stringconst();
-	  if(s=="sensitive")
+	  if(s==String("sensitive"))
 	    hotkeycase=1;
-	  else if (s=="insensitive")
+	  else if (s==String("insensitive"))
 	    hotkeycase=0;
 	  else {
 	    cerr<<"install-menus hotkeycase can only be {,in}sensitive"<<endl;
@@ -748,8 +754,18 @@ void configinfo::check_config(){
   }
 }
 String configinfo::prefix(){
-  if(getuid())
-    return String(getenv("HOME"))+"/"+userprefix();
+  DIR *d;
+  String s;
+
+  if(getuid()){
+    s=String(getenv("HOME"))+"/"+userprefix();
+    d=opendir(s);
+    if(d){
+      closedir(d);
+      return s;
+    } else
+      return rootpref;
+  }
   else
     return rootpref;
 }
@@ -968,7 +984,7 @@ void check_vars(parsestream &i,
 
   for(k=0;k<need.size();k++){
     j=m.find(need[k]);
-    if((j==m.end())||((*j).second==""))
+    if((j==m.end())||((*j).second==String("")))
       throw missing_tag(&i,need[k]);
   }
 }
@@ -980,7 +996,7 @@ void read_input(parsestream &i){
       String sec;
       m=read_vars(i);
       check_vars(i,m);
-      sec=m[SECTION_VAR]+'/';
+      sec=m[SECTION_VAR]+String('/');
       if(m.find(SORT_VAR)!=m.end())
 	sec=sec+m[SORT_VAR]+":";
       sec=sec+replace(m[TITLE_VAR],'/','_');
@@ -1076,7 +1092,7 @@ int main(int argc, char **argv){
       system(config->prerun->soutput(menu.vars));
     parsestream psscript(cin);
     read_input(psscript);
-    if(menu.vars[TITLE_VAR]=="")
+    if(menu.vars[TITLE_VAR]==String(""))
       menu.vars[TITLE_VAR]=config->mainmenutitle();
     menu.postprocess();
     menu.output();
