@@ -60,7 +60,7 @@ bool menuentry::check_install(parsestream &i){
   String name, val;
 
   name=i.get_name();
-  if(name!=COND_PACKAGE)
+  if(name!=String(COND_PACKAGE))
     throw unknown_cond_package(&i);
   i.skip_char('(');
   val=i.get_name();
@@ -136,7 +136,7 @@ void menuentry::output(vector<String> &s){
 void menuentry::output_compat(vector<String> &s){
   String t,icon;
   icon=data[ICON_VAR];
-  if(icon=="")
+  if(icon==String(""))
     icon="none";
   t=data[NEEDS_VAR]  +" "
    +data[SECTION_VAR]+" id"
@@ -166,51 +166,51 @@ configinfo::configinfo(){
 void configinfo::parse_def(const String &var, parsestream &p){
   String s;
   s=p.get_Stringconst();
-  if(var=="verbosity"){
-    if     (s=="quiet")   verbosity=report_quiet;
-    else if(s=="normal")  verbosity=report_normal;
-    else if(s=="verbose") verbosity=report_verbose;
-    else if(s=="debug")   verbosity=report_debug;
+  if(var==String("verbosity")){
+    if     (s==String("quiet"))   verbosity=report_quiet;
+    else if(s==String("normal"))  verbosity=report_normal;
+    else if(s==String("verbose")) verbosity=report_verbose;
+    else if(s==String("debug"))   verbosity=report_debug;
     else throw def_error();
-  } else if(var=="method"){
-    if     (s=="stdout")  method=method_stdout;
-    else if(s=="stderr")  method=method_stderr;
-    else if(s=="syslog") { 
+  } else if(var==String("method")){
+    if     (s==String("stdout"))  method=method_stdout;
+    else if(s==String("stderr"))  method=method_stderr;
+    else if(s==String("syslog")) { 
       method=method_syslog;
       String facility;
       String priority;
       facility=p.get_Stringconst();
       p.skip_char('.');
       priority=p.get_Stringconst();
-           if(facility=="auth")       syslog_facility=LOG_AUTH;
-      else if(facility=="authpriv")   syslog_facility=LOG_AUTHPRIV;
-      else if(facility=="authcron")   syslog_facility=LOG_CRON;
-      else if(facility=="authdaemon") syslog_facility=LOG_AUTHPRIV;
-      else if(facility=="authkern")   syslog_facility=LOG_KERN;
-      else if(facility=="authlocal0") syslog_facility=LOG_LOCAL0;
-      else if(facility=="authlocal1") syslog_facility=LOG_LOCAL1;
-      else if(facility=="authlocal2") syslog_facility=LOG_LOCAL2;
-      else if(facility=="authlocal3") syslog_facility=LOG_LOCAL3;
-      else if(facility=="authlocal4") syslog_facility=LOG_LOCAL4;
-      else if(facility=="authlocal5") syslog_facility=LOG_LOCAL5;
-      else if(facility=="authlocal6") syslog_facility=LOG_LOCAL6;
-      else if(facility=="authlocal7") syslog_facility=LOG_LOCAL7;
-      else if(facility=="authlpr")    syslog_facility=LOG_LPR;
-      else if(facility=="authmail")   syslog_facility=LOG_MAIL;
-      else if(facility=="authnews")   syslog_facility=LOG_NEWS;
-      else if(facility=="authsyslog") syslog_facility=LOG_SYSLOG;
-      else if(facility=="authuser")   syslog_facility=LOG_USER;
-      else if(facility=="authuucp")   syslog_facility=LOG_UUCP;
+           if(facility==String("auth"))       syslog_facility=LOG_AUTH;
+      else if(facility==String("authpriv"))   syslog_facility=LOG_AUTHPRIV;
+      else if(facility==String("authcron"))   syslog_facility=LOG_CRON;
+      else if(facility==String("authdaemon")) syslog_facility=LOG_AUTHPRIV;
+      else if(facility==String("authkern"))   syslog_facility=LOG_KERN;
+      else if(facility==String("authlocal0")) syslog_facility=LOG_LOCAL0;
+      else if(facility==String("authlocal1")) syslog_facility=LOG_LOCAL1;
+      else if(facility==String("authlocal2")) syslog_facility=LOG_LOCAL2;
+      else if(facility==String("authlocal3")) syslog_facility=LOG_LOCAL3;
+      else if(facility==String("authlocal4")) syslog_facility=LOG_LOCAL4;
+      else if(facility==String("authlocal5")) syslog_facility=LOG_LOCAL5;
+      else if(facility==String("authlocal6")) syslog_facility=LOG_LOCAL6;
+      else if(facility==String("authlocal7")) syslog_facility=LOG_LOCAL7;
+      else if(facility==String("authlpr"))    syslog_facility=LOG_LPR;
+      else if(facility==String("authmail"))   syslog_facility=LOG_MAIL;
+      else if(facility==String("authnews"))   syslog_facility=LOG_NEWS;
+      else if(facility==String("authsyslog")) syslog_facility=LOG_SYSLOG;
+      else if(facility==String("authuser"))   syslog_facility=LOG_USER;
+      else if(facility==String("authuucp"))   syslog_facility=LOG_UUCP;
       else throw def_error();
 
-	   if(priority=="emerg")      syslog_priority=LOG_EMERG;
-      else if(priority=="alert")      syslog_priority=LOG_ALERT;
-      else if(priority=="crit")       syslog_priority=LOG_CRIT;
-      else if(priority=="err")        syslog_priority=LOG_ERR;
-      else if(priority=="warning")    syslog_priority=LOG_WARNING;
-      else if(priority=="notice")     syslog_priority=LOG_NOTICE;
-      else if(priority=="info")       syslog_priority=LOG_INFO;
-      else if(priority=="debug")      syslog_priority=LOG_DEBUG;
+	   if(priority==String("emerg"))      syslog_priority=LOG_EMERG;
+      else if(priority==String("alert"))      syslog_priority=LOG_ALERT;
+      else if(priority==String("crit"))       syslog_priority=LOG_CRIT;
+      else if(priority==String("err"))        syslog_priority=LOG_ERR;
+      else if(priority==String("warning"))    syslog_priority=LOG_WARNING;
+      else if(priority==String("notice"))     syslog_priority=LOG_NOTICE;
+      else if(priority==String("info"))       syslog_priority=LOG_INFO;
+      else if(priority==String("debug"))      syslog_priority=LOG_DEBUG;
       else throw def_error();
     }
     else throw def_error();
@@ -259,6 +259,9 @@ trans_class::trans_class(const String &m,
   replace_var=rv;
 }
 
+trans_class::~trans_class(){
+  
+}
 bool  trans_class::check(String &s){
   config.report(String("checking ")+match+" < "+s,configinfo::report_debug);
   return match.contains(s,0);
@@ -318,7 +321,7 @@ void translateinfo::init(parsestream &i){
 
       i.skip_space();
       match=i.get_Stringconst();
-      if(match==ENDTRANSLATE_TRANS){
+      if(match==String(ENDTRANSLATE_TRANS)){
 	i.skip_line();
 	break;
       }
@@ -402,43 +405,27 @@ void translateinfo::debuginfo(){
 //  Installed Package Status:
 //
 
-void read_pkginfo (const char *statusfile)
-  //this procedure written by Tom Lees, originally in pure C.
+void read_pkginfo (void)
 {
   FILE *status;
-  char tmp[MAX_LINE], tmp2[MAX_LINE] = "", *s, *s2;
-  int i;
-  
-  if(!(status = fopen (statusfile, "r")))
-    throw ferror_read(statusfile);
+  char tmp[MAX_LINE];
+  char *getselections=
+    "dpkg --get-selections|sed -n -e 's/[ \t]*\\(install\\|hold\\)$//p'";
+
+  status=popen(getselections,"r");
+
+  if(!status)
+    throw ferror_read(getselections);
+
   config.report(String("Reading installed packages..."),
 		configinfo::report_verbose);
   while (!feof (status)){
     fgets (tmp, MAX_LINE, status);
-    if (strlen (tmp) == 0)
-      continue;
-    if (tmp[strlen (tmp) - 1] == '\n')
-      tmp[strlen (tmp) - 1] = '\0';
-    for (i = strlen (tmp); i >= 0 && tmp[i] == ' '; i--) ;
-    tmp[i] = '\0';
-    for (s = tmp; *s == ' '; s++) ;
-    if (strlen (s) == 0)
-      continue;
-    
-    if (!strncmp (s, "Package: ", 9))
-	strcpy (tmp2, s + 9);
-    else if (!strncmp (s, "Status: ", 8) && strlen (tmp2) != 0) {
-      for (s2 = s + 8; *s2 == ' '; s2++) ;
-      for (; *s2 != ' ' && *s2 != '\0'; s2++) ;
-      for (; *s2 == ' '; s2++) ;
-      if (strncmp (s2, "ok ", 3))
-	continue;
-      for (s2 += 3; *s2 == ' '; s2++) ;
-      if (!strcmp (s2, "installed"))
-	installed_packages.insert(tmp2);
-    }
+    if(tmp[strlen(tmp)-1]=='\n')
+      tmp[strlen(tmp)-1]=0;
+    installed_packages.insert(tmp);
   }
-  fclose (status);
+  pclose (status);
 }
 
 void read_menufile(const String &filename,
@@ -456,7 +443,15 @@ void read_menufile(const String &filename,
     if(executable(filename)){
       pr.open(filename,ios::in);
       pipe_istr=new istream(&pr);
-      i=new parsestream(*pipe_istr);
+      try{
+	i=new parsestream(*pipe_istr);
+      } catch (endoffile d){
+	cerr<<"Error (or no input available from stdout) while executing "<<endl
+	    <<filename<<" . Note that it is"<<endl
+	    <<"a _feature_ of menu that it executes menuentryfiles that have"<<endl
+	    <<"the executable bit set. See the documentation."<<endl;
+	throw endoffile(d);
+      }
     }
     else
       i=new parsestream(filename);
@@ -498,7 +493,7 @@ void read_menufilesdir(const String &dirname,
     dir=open_dir_check(dirname);
     while((entry=readdir(dir))){
       name=String(entry->d_name);
-      if((name!="README")&&(name!="core")&&(name[0]!='.'))
+      if((name!=String("README"))&&(name!=String("core"))&&(name[0]!='.'))
 	if(menufiles_processed.find(name)==menufiles_processed.end()){
 	  menufiles_processed.insert(name);
 	  name=dirname+name;
@@ -627,7 +622,7 @@ void create_lock(){
     if(!of.good()){
       config.report("Update-menu: cannot write to lockfile "
 		    UPMEN_LOCKFILE". Aborting.",
-		    configinfo::report_normal);
+		    configinfo::report_quiet);
       exit(1);
     }
   }
@@ -647,9 +642,10 @@ int check_dpkglock(){
   struct flock fl;
   char buf[MAX_LINE];
   if(getuid()){
-    if(verbose)
-      cerr<<"update-menus run by user -- cannot determine if dpkg is locking"<<endl
-	  <<DPKG_LOCKFILE<<": assuming there is no lock"<<endl;
+    config.report("update-menus run by user -- cannot determine if dpkg is "
+		  "locking "DPKG_LOCKFILE": assuming there is no lock",
+		  configinfo::report_verbose);
+    
     return 0;
   }
   fd=open(DPKG_LOCKFILE, O_RDWR|O_CREAT|O_TRUNC, 0660);
@@ -663,8 +659,7 @@ int check_dpkglock(){
     close(fd);
     if (errno == EWOULDBLOCK || errno == EAGAIN || errno == EACCES)
       return 1;
-    cerr<<"update-menus: Thank you for installing this test version of the menu package."<<endl
-	<<"update-menus: Unfortunately, I encountered an unknown errno (="
+    cerr<<"update-menus: Encountered an unknown errno (="
 	<<errno<<")."<<endl
 	<<"update-menus: Could you please be so kind as to email joostje@debian.org"<<endl
 	<<"update-menus: the errno (="<<errno<<") with a discription of what you did to"<<endl
@@ -694,33 +689,34 @@ void wait_dpkg(){
   other=kill_other_updatemenus();
 
   if(check_dpkglock()){
-    if((child=fork())){
+    config.report("Update-menus: waiting for dpkg to finish"
+		    "(forking to background)\n"
+		  "Update-menus: (checking "DPKG_LOCKFILE")",
+		  configinfo::report_normal);
+    if((child=fork()))
       exit(0);
-    }else{
+    else{
       create_lock();
-      if((!other)||verbose){
-	cerr<<"Update-menus: waiting for dpkg to finish (forking to background)"<<endl
-	    <<"Update-menus: (checking "DPKG_LOCKFILE")"<<endl;
-      }
       while(check_dpkglock())
 	sleep(2);
-      if(debug)
-	cerr<<"Update-menus: Dpkg lock on lockfile disapeared, generate menufiles"<<endl;
+      config.report("Update-menus: Dpkg lock on lockfile disapeared, "
+		    "generate menufiles",
+		    configinfo::report_debug);
     }
   } else{
     create_lock();
-    if(verbose)
-      cerr<<"Update-menus: Dpkg not locking dpkg status area. Good."<<endl;
+    config.report("Update-menus: Dpkg not locking dpkg status area. Good.",
+		  configinfo::report_verbose);
   }
 }
 
 void parse_params(char **argv){
   while(*(++argv)){
-    if(String("-d")==*argv)
+    if(String("-d")==String(*argv))
       config.set_verbosity(configinfo::report_debug);
-    if(String("-v")==*argv)
+    if(String("-v")==String(*argv))
       config.set_verbosity(configinfo::report_verbose);
-    if(String("-h")==*argv){
+    if(String("-h")==String(*argv)){
       cerr<<"update-menus: update the various window-manager config files (and"<<endl
 	  <<"  dwww, and pdmenu) after menuentry files were changed."<<endl
 	  <<"Usage: update-menus [options] "<<endl
@@ -772,7 +768,7 @@ main (int, char **argv){
     read_rootconfiginfo();
     parse_params(argv);
     wait_dpkg();
-    read_pkginfo(DPKG_STATUSFILE);
+    read_pkginfo();
     transinfo=NULL;
 
     read_usertranslateinfo();
